/**
 * @file
 * CKEditor Accordion functionality.
 */

(function (Drupal, drupalSettings) {
  'use strict';
  Drupal.behaviors.ckeditorAccordion = {
      /* SLIDE UP */
      slideUp: function(target, duration=300) {
        target.style.transitionProperty = 'height, margin, padding';
        target.style.transitionDuration = duration + 'ms';
        target.style.boxSizing = 'border-box';
        target.style.height = target.offsetHeight + 'px';
        target.offsetHeight;
        target.style.overflow = 'hidden';
        target.style.height = 0;
        target.style.paddingTop = 0;
        target.style.paddingBottom = 0;
        target.style.marginTop = 0;
        target.style.marginBottom = 0;
        window.setTimeout( () => {
              target.style.display = 'none';
              target.style.removeProperty('height');
              target.style.removeProperty('padding-top');
              target.style.removeProperty('padding-bottom');
              target.style.removeProperty('margin-top');
              target.style.removeProperty('margin-bottom');
              target.style.removeProperty('overflow');
              target.style.removeProperty('transition-duration');
              target.style.removeProperty('transition-property');
              //alert("!");
        }, duration);
    },

    /* SLIDE DOWN */
    slideDown: function(target, duration=300) {
        target.style.removeProperty('display');
        let display = window.getComputedStyle(target).display;
        if (display === 'none') display = 'block';
        target.style.display = display;
        let height = target.offsetHeight;
        target.style.overflow = 'hidden';
        target.style.height = 0;
        target.style.paddingTop = 0;
        target.style.paddingBottom = 0;
        target.style.marginTop = 0;
        target.style.marginBottom = 0;
        target.offsetHeight;
        target.style.boxSizing = 'border-box';
        target.style.transitionProperty = "height, margin, padding";
        target.style.transitionDuration = duration + 'ms';
        target.style.height = height + 'px';
        target.style.removeProperty('padding-top');
        target.style.removeProperty('padding-bottom');
        target.style.removeProperty('margin-top');
        target.style.removeProperty('margin-bottom');
        window.setTimeout( () => {
          target.style.removeProperty('height');
          target.style.removeProperty('overflow');
          target.style.removeProperty('transition-duration');
          target.style.removeProperty('transition-property');
        }, duration);
    },

    attach: function (context, settings) {

      // Create accordion functionality if the required elements exist is available.
      let $ckeditorAccordions = document.getElementsByClassName('ckeditor-accordion');
      if ($ckeditorAccordions.length > 0) {
        // Create simple accordion mechanism for each tab.
        for (let i = 0; i < $ckeditorAccordions.length; i++) {
          let $accordion = $ckeditorAccordions[i];
          if ($accordion.classList.contains('styled')) {
            return;
          }

          // The first one is the correct one.
          if (!drupalSettings.ckeditorAccordion.accordionStyle.collapseAll) {
            $accordion.querySelector('dt:first-child').classList.add('active');
            let dd = $accordion.querySelector('dd:first-of-type');
            dd.classList.add('active');
            dd.style.display = 'block';
          }

          // Turn the accordion tabs to links so that the content is accessible & can be traversed using keyboard.
          let childDts = Array.from($accordion.children).filter(function (child) {
            return (child.tagName.toLowerCase() == 'dt');
          });

          for (let x = 0; x < childDts.length; x++) {
            var $tab = childDts[x];
            var tabText = $tab.innerText.trim();
            var toggleClass = $tab.classList.contains('active') ? ' active' : '';
            $tab.innerHTML = '<a class="ckeditor-accordion-toggler" href="#"><span class="ckeditor-accordion-toggle' + toggleClass + '"></span>' + tabText + '</a>';
          }

          // Wrap the accordion in a div element so that quick edit function shows the source correctly.
          $accordion.classList.add('styled');
          $accordion.classList.remove('ckeditor-accordion');
          let $wrapper = document.createElement('div');
          $wrapper.classList.add('ckeditor-accordion-container');
          $accordion.parentNode.appendChild($wrapper);
          $wrapper.appendChild($accordion);

          // Trigger an ckeditorAccordionAttached event to let other frameworks know that the accordion has been attached.
          const eventAccordionAttached = new Event('ckeditorAccordionAttached');
          $accordion.dispatchEvent(eventAccordionAttached);
        }

        // Add click event to body once because quick edits & ajax calls might reset the HTML.
        const $accordionTogglers = once('ckeditorAccordionToggleEvent', 'body .ckeditor-accordion-toggler', context);
        for (let x = 0; x < $accordionTogglers.length; x++) {
          $accordionTogglers[x].addEventListener("click", function (e) {
            console.log(this, e)
            var $t = this.parentNode;
            var $parent = $t.parentNode;

            // Clicking on open element, close it.
            if ($t.classList.contains('active')) {
              $t.classList.remove('active');
              Drupal.behaviors.ckeditorAccordion.slideUp($t.nextSibling);
              $t.nextSibling.classList.remove('active');
            }
            else {
              if(!drupalSettings.ckeditorAccordion.accordionStyle.keepRowsOpen) {
                let activeChildDds = Array.from($parent.children).filter(function (child) {
                  return (child.tagName.toLowerCase() == 'dd' && child.classList.contains('active'));
                });
                activeChildDds.forEach(function(childToSlideUp) {
                  Drupal.behaviors.ckeditorAccordion.slideUp(childToSlideUp);
                  childToSlideUp.classList.remove('active');
                });
                if($parent.querySelector('.active') != null) {
                  $parent.querySelector('.active').classList.remove('active');
                }
              }

              // Show the selected tab.
              $t.classList.add('active');
              Drupal.behaviors.ckeditorAccordion.slideDown($t.nextSibling)
              $t.nextSibling.classList.add('active');
            }

            // Don't add hash to url.
            e.preventDefault();
          });
        }

      }
    }
  };
})(Drupal, drupalSettings);
